@@ -5,7 +5,7 @@ import io
 import itertools
 from os import PathLike
 from pathlib import Path
-from typing import Any, Iterable, List, TextIO, Union
+from typing import Iterable, List, TextIO, Union
 
 import geopandas
 from geopandas import GeoDataFrame
@@ -29,9 +29,9 @@ ATCF_FIELDS = {
     # YYYYMMDDHH - Warning Date-Time-Group: 0000010100 through 9999123123. (note, 4 digit year)
     'YYYYMMDDHH': 'datetime',
     # TECHNUM/MIN - objective technique sorting number, minutes for best track: 00 - 99
-    'TECHNUM/MIN': 'record_type_number',
+    'TECHNUM/MIN': 'advisory_number',
     # TECH - acronym for each objective technique or CARQ or WRNG, BEST for best track.
-    'TECH': 'record_type',
+    'TECH': 'advisory',
     # TAU - forecast period: -24 through 240 hours, 0 for best-track, negative taus used for CARQ and WRNG records.
     'TAU': 'forecast_hours',
     # LatN/S - Latitude (tenths of degrees) for the DTG: 0 through 900, N/S is the hemispheric index.
@@ -160,7 +160,7 @@ def atcf_files(
     if not isinstance(mode, ATCF_Mode):
         mode = typepigeon.convert_value(mode, ATCF_Mode)
 
-    if mode == ATCF_Mode.historical and year is None or isinstance(year, Iterable):
+    if mode == ATCF_Mode.HISTORICAL and year is None or isinstance(year, Iterable):
         if year is None:
             year = range(ATCF_RECORD_START_YEAR, datetime.today().year + 1)
         return list(
@@ -199,17 +199,17 @@ class ATCF_FileDeck(Enum):
 
 
 class ATCF_Mode(Enum):
-    historical = 'ARCHIVE'
-    realtime = 'aid_public'
+    HISTORICAL = 'ARCHIVE'
+    REALTIME = 'aid_public'
 
 
-class ATCF_RecordType(Enum):
-    best = 'BEST'
-    ofcl = 'OFCL'
-    ofcp = 'OFCP'
-    hmon = 'HMON'
-    carq = 'CARQ'
-    hwrf = 'HWRF'
+class ATCF_Advisory(Enum):
+    BEST = 'BEST'
+    OFCL = 'OFCL'
+    OFCP = 'OFCP'
+    HMON = 'HMON'
+    CARQ = 'CARQ'
+    HWRF = 'HWRF'
 
 
 def get_atcf_entry(
@@ -254,9 +254,9 @@ def atcf_url(
             raise ValueError('NHC storm code not given')
         entry = get_atcf_entry(basin=nhc_code[:2], storm_number=int(nhc_code[2:4]), year=year)
         if entry['source'] == 'ARCHIVE':
-            mode = ATCF_Mode.historical
+            mode = ATCF_Mode.HISTORICAL
         else:
-            mode = ATCF_Mode.realtime
+            mode = ATCF_Mode.REALTIME
 
     if not isinstance(file_deck, ATCF_FileDeck):
         try:
@@ -272,7 +272,7 @@ def atcf_url(
         except ValueError:
             mode = None
 
-    if mode == ATCF_Mode.historical:
+    if mode == ATCF_Mode.HISTORICAL:
         if year is None:
             raise ValueError('NHC storm code not given')
         nhc_dir = f'archive/{year}'
@@ -298,36 +298,22 @@ def atcf_url(
     return url
 
 
-def normalize_atcf_value(value: Any, to_type: type, round_digits: int = None,) -> Any:
-    if type(value).__name__ == 'Quantity':
-        value = value.magnitude
-    if not (value is None or pandas.isna(value) or value == ''):
-        if round_digits is not None and issubclass(to_type, (int, float)):
-            if isinstance(value, str):
-                value = float(value)
-            value = round(value, round_digits)
-        value = typepigeon.convert_value(value, to_type)
-    return value
-
-
 def read_atcf(
     atcf: Union[PathLike, io.BytesIO, TextIO],
-    record_types: List[ATCF_RecordType] = None,
+    advisories: List[ATCF_Advisory] = None,
     fort_22: bool = False,
 ) -> GeoDataFrame:
     """
     read ATCF format
 
     :param atcf: path or buffered reader
-    :param record_types: allowed record types
+    :param advisories: allowed advisory types
     :param fort_22: whether to parse `fort.22` fields
     :return: data frame of parsed ATCF data
     """
 
-    if record_types is not None:
-        record_types = [
-            typepigeon.convert_value(record_type, str) for record_type in record_types
-        ]
+    if advisories is not None:
+        advisories = [typepigeon.convert_value(advisory, str) for advisory in advisories]
 
     if isinstance(atcf, (str, PathLike, Path)):
         atcf = open(atcf)
@@ -342,11 +328,19 @@ def read_atcf(
         for line in lines
     )
 
-    data = DataFrame.from_records(lines, columns=list(ATCF_FIELDS),).astype(
-        {field: 'string' for field in ATCF_FIELDS}
+    data = DataFrame.from_records(lines)
+    data.rename(
+        columns={index: list(ATCF_FIELDS)[index] for index in range(len(data.columns))},
+        inplace=True,
+    )
+    for column in ATCF_FIELDS:
+        if column not in data.columns:
+            data[column] = pandas.NA
+    data.astype(
+        {field: 'string' for field in data.columns}, copy=False,
     )
 
-    if data['USERDEFINED'].str.contains(',').any():
+    if 'USERDEFINED' in data and data['USERDEFINED'].str.contains(',').any():
         if fort_22:
             extra_fields = FORT_22_FIELDS
         else:
@@ -368,10 +362,10 @@ def read_atcf(
         except ValueError:
             pass
 
-    if record_types is not None:
-        data = data[data['TECH'].isin(record_types)]
+    if advisories is not None and len(advisories) > 0:
+        data = data[data['TECH'].isin(advisories)]
         if len(data) == 0:
-            raise ValueError(f'no ATCF records found matching "{record_types}"')
+            raise ValueError(f'no ATCF records found matching "{advisories}"')
 
     best_track_records = (data['TECH'] == 'BEST') & (
         data.loc[data['TECH'] == 'BEST', 'TECHNUM/MIN'].str.strip().str.len() > 0
@@ -430,7 +424,22 @@ def read_atcf(
 
     data[float_fields] = data[float_fields].astype(float, copy=False)
 
-    data.rename(columns=ATCF_FIELDS, inplace=True)
+    data.rename(
+        columns={
+            **{field: value for field, value in ATCF_FIELDS.items() if field in data.columns},
+            **{
+                field: value
+                for field, value in EXTRA_ATCF_FIELDS.items()
+                if field in data.columns
+            },
+            **{
+                field: value
+                for field, value in FORT_22_FIELDS.items()
+                if field in data.columns
+            },
+        },
+        inplace=True,
+    )
 
     return GeoDataFrame(
         data, geometry=geopandas.points_from_xy(data['longitude'], data['latitude'],)
