@@ -1,12 +1,13 @@
 from datetime import datetime
 from functools import lru_cache
 from os import PathLike
+from typing import List
 
 import pandas
 from shapely import ops
 from shapely.geometry import MultiPolygon, Polygon
 from shapely.geometry.base import BaseGeometry
-import typepigeon
+from shapely.ops import shape as shapely_shape
 import xarray
 from xarray import Dataset
 
@@ -15,14 +16,14 @@ from stormevents.coops.tidalstations import (
     COOPS_Product,
     COOPS_Station,
     coops_stations_within_region,
-    COOPS_StationType,
+    COOPS_StationStatus,
     COOPS_TidalDatum,
     COOPS_TimeZone,
     COOPS_Units,
 )
 from stormevents.nhc import nhc_storms, VortexTrack
-from stormevents.nhc.atcf import ATCF_FileDeck, ATCF_Mode
-from stormevents.usgs import StormFloodEvent, usgs_flood_storms
+from stormevents.nhc.atcf import ATCF_Advisory, ATCF_FileDeck, ATCF_Mode
+from stormevents.usgs import usgs_flood_storms, USGS_StormEvent
 from stormevents.utilities import relative_to_time_interval, subset_time_interval
 
 
@@ -42,19 +43,19 @@ class StormEvent:
         :param end_date: ending time
 
         >>> StormEvent('florence', 2018)
-        StormEvent('FLORENCE', 2018)
+        StormEvent(name='FLORENCE', year=2018, start_date=Timestamp('2018-08-30 06:00:00'), end_date=Timestamp('2018-09-18 12:00:00'))
 
         >>> StormEvent('paine', 2016, start_date='2016-09-18', end_date=datetime(2016, 9, 19, 12))
-        StormEvent('PAINE', 2016, start_date='2016-09-19 00:00:00', end_date='2016-09-19 12:00:00')
+        StormEvent(name='PAINE', year=2016, start_date=Timestamp('2016-09-18 00:00:00'), end_date=datetime.datetime(2016, 9, 19, 12, 0))
 
         >>> StormEvent('florence', 2018, start_date=timedelta(days=2))
-        StormEvent('FLORENCE', 2018, start_date='2018-09-01 06:00:00')
+        StormEvent(name='FLORENCE', year=2018, start_date=Timestamp('2018-09-01 06:00:00'), end_date=Timestamp('2018-09-18 12:00:00'))
 
         >>> StormEvent('henri', 2021, start_date=timedelta(days=-3), end_date=timedelta(days=-2))
-        StormEvent('HENRI', 2021, start_date='2021-08-21 12:00:00', end_date='2021-08-22 12:00:00')
+        StormEvent(name='HENRI', year=2021, start_date=Timestamp('2021-08-21 12:00:00'), end_date=Timestamp('2021-08-22 12:00:00'))
 
         >>> StormEvent('ida', 2021, end_date=timedelta(days=2))
-        StormEvent('IDA', 2021, end_date='2021-08-29 18:00:00')
+        StormEvent(name='IDA', year=2021, start_date=Timestamp('2021-08-27 18:00:00'), end_date=Timestamp('2021-08-29 18:00:00'))
         """
 
         storms = nhc_storms(year=year)
@@ -84,7 +85,7 @@ class StormEvent:
         :return: storm object
 
         >>> StormEvent.from_nhc_code('EP172016')
-        StormEvent('PAINE', 2016)
+        StormEvent(name='PAINE', year=2016, start_date=Timestamp('2016-09-18 00:00:00'), end_date=Timestamp('2016-09-21 12:00:00'))
         """
 
         try:
@@ -118,7 +119,7 @@ class StormEvent:
         :return: storm object
 
         >>> StormEvent.from_usgs_id(310)
-        StormEvent('HENRI', 2021, end_date='2021-08-24 12:00:00')
+        StormEvent(name='HENRI', year=2021, start_date=Timestamp('2021-08-20 18:00:00'), end_date=Timestamp('2021-08-24 12:00:00'))
         """
 
         flood_events = usgs_flood_storms(year=year)
@@ -236,14 +237,12 @@ class StormEvent:
             data_end = VortexTrack.from_storm_name(self.name, self.year).end_date
         return data_end
 
-    @lru_cache(maxsize=None)
     def track(
         self,
         start_date: datetime = None,
         end_date: datetime = None,
         file_deck: ATCF_FileDeck = None,
-        mode: ATCF_Mode = None,
-        record_type: str = None,
+        advisories: List[ATCF_Advisory] = None,
         filename: PathLike = None,
     ) -> VortexTrack:
         """
@@ -252,14 +251,13 @@ class StormEvent:
         :param start_date: start date
         :param end_date: end date
         :param file_deck: ATCF file deck
-        :param mode: ATCF mode
-        :param record_type: ATCF record type
+        :param advisories: ATCF advisory types
         :param filename: file path to ``fort.22``
         :return: vortex track
 
         >>> storm = StormEvent('florence', 2018)
-        >>> storm.track(file_deck='b')
-        VortexTrack('AL062018', Timestamp('2018-08-30 06:00:00'), Timestamp('2018-09-18 12:00:00'), <ATCF_FileDeck.BEST: 'b'>, <ATCF_Mode.historical: 'ARCHIVE'>, 'BEST', None)
+        >>> storm.track()
+        VortexTrack('AL062018', Timestamp('2018-08-30 06:00:00'), Timestamp('2018-09-18 12:00:00'), <ATCF_FileDeck.BEST: 'b'>, <ATCF_Mode.HISTORICAL: 'ARCHIVE'>, [<ATCF_Advisory.BEST: 'BEST'>], None)
         """
 
         if start_date is None:
@@ -276,13 +274,12 @@ class StormEvent:
                 start_date=start_date,
                 end_date=end_date,
                 file_deck=file_deck,
-                mode=mode,
-                record_type=record_type,
+                advisories=advisories,
             )
         return track
 
     @property
-    def flood_event(self) -> StormFloodEvent:
+    def flood_event(self) -> USGS_StormEvent:
         """
         :return: USGS high-water marks (HWMs) for this storm event
 
@@ -307,14 +304,15 @@ class StormEvent:
 
         configuration = {'name': self.name, 'year': self.year}
         if self.__high_water_marks is None or configuration != self.__previous_configuration:
-            self.__high_water_marks = StormFloodEvent(name=self.name, year=self.year)
+            self.__high_water_marks = USGS_StormEvent(name=self.name, year=self.year)
         return self.__high_water_marks
 
     def coops_product_within_isotach(
         self,
         product: COOPS_Product,
         wind_speed: int,
-        station_type: COOPS_StationType = None,
+        advisories: List[ATCF_Advisory] = None,
+        station_type: COOPS_StationStatus = None,
         start_date: datetime = None,
         end_date: datetime = None,
         datum: COOPS_TidalDatum = None,
@@ -328,6 +326,7 @@ class StormEvent:
 
         :param product: CO-OPS product
         :param wind_speed: wind speed in knots (one of ``34``, ``50``, or ``64``)
+        :param advisories: ATCF advisory types
         :param start_date: start date
         :param end_date: end date
         :param station_type: either ``current`` or ``historical``
@@ -358,10 +357,17 @@ class StormEvent:
         if isinstance(track, VortexTrack):
             track.start_date = start_date
             track.end_date = end_date
+            track.advisories = advisories
         else:
-            track = self.track(start_date=start_date, end_date=end_date, filename=track)
+            track = self.track(
+                start_date=start_date, end_date=end_date, filename=track, advisories=advisories
+            )
 
-        region = ops.unary_union(list(track.wind_swaths(wind_speed).values()))
+        polygons = []
+        wind_swaths = track.wind_swaths(wind_speed)
+        for advisory, advisory_wind_swaths in wind_swaths.items():
+            polygons.append(ops.unary_union(list(advisory_wind_swaths.values())))
+        region = ops.unary_union(polygons)
 
         return self.coops_product_within_region(
             region=region,
@@ -381,7 +387,7 @@ class StormEvent:
         region: Polygon,
         start_date: datetime = None,
         end_date: datetime = None,
-        station_type: COOPS_StationType = None,
+        station_type: COOPS_StationStatus = None,
         datum: COOPS_TidalDatum = None,
         units: COOPS_Units = None,
         time_zone: COOPS_TimeZone = None,
@@ -403,7 +409,7 @@ class StormEvent:
 
         >>> import shapely
         >>> storm = StormEvent('florence', 2018)
-        >>> region = shapely.geometry.box(*storm.track().linestring.bounds)
+        >>> region = shapely.geometry.box(*storm.track().linestrings.bounds)
         >>> storm.coops_product_within_region('water_level', region=region, start_date='2018-09-12 14:03:00', end_date='2018-09-14')
         <xarray.Dataset>
         Dimensions:  (nos_id: 89, t: 340)
@@ -421,10 +427,7 @@ class StormEvent:
         """
 
         if not isinstance(region, BaseGeometry):
-            try:
-                region = typepigeon.convert_value(region, MultiPolygon)
-            except ValueError:
-                region = typepigeon.convert_value(region, Polygon)
+            region = shapely_shape(region)
 
         if start_date is None:
             start_date = self.start_date
@@ -439,7 +442,7 @@ class StormEvent:
                 start=self.start_date, end=self.end_date, relative=end_date
             )
 
-        stations = coops_stations_within_region(region=region, station_type=station_type)
+        stations = coops_stations_within_region(region=region, station_status=station_type)
 
         if len(stations) > 0:
             stations_data = []
